/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to you under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.shale.usecases.rolodex;

import java.io.IOException;
import java.net.URLEncoder;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.component.UIData;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;

import org.apache.shale.util.Messages;

public class HeaderSorterRenderer extends Renderer {

    
    /**
     * <p>Localized messages for this application.</p>
     */
    private static Messages messages =
      new Messages("org.apache.shale.usecases.view.Bundle");
    

    /**
     * <p>Constant used to create the query parameter that
     * identifies the sorted column.</p>
     */
    private static final String SORTED_COLUMN = "s";

    
    /**
     * <p>Constant used to create the query parameter that
     * identifies the sorted column's next sort direction.</p>
     */
    private static final String SORTED_SEQUENCE = "a";

    
    
    /**
     * <p>Finds the parent IUData component.</p>
     */
    protected UIData findDataComponent(UIComponent uicomponent) {
        return (UIData) findDataParent(uicomponent);
    }

    /**
     * <p>Recursive method that finds the first parent Idata component.</p>
     */
    private UIComponent findDataParent(UIComponent uicomponent) {
        if (uicomponent == null)
            return null;
        if (uicomponent instanceof UIData)
            return uicomponent;
        else
            return findDataParent(uicomponent.getParent());
    }

    /**
     * <p>Returns the base context root of the form using the
     * view handler.</p>
     */
    protected String getActionStr(FacesContext context) {
        String uri = context.getViewRoot().getViewId();
        String url =
            context.getApplication().getViewHandler().getActionURL(
                context,
                uri);

        return url;
    }

    /**
     * <p>The name of the query parameter used to control column sorting.</p>
     */
    protected String getSortedTag(FacesContext context, UIData data) {
        StringBuffer clientId = new StringBuffer(data.getClientId(context));
        clientId.append(SORTED_COLUMN);
        return clientId.toString();
    }

    /**
     * <p>Determines if the column was click to be sorted on.</p>
     */
    protected boolean wasSelected(
        FacesContext context,
        UIData data,
        HeaderSorter column) {

        boolean wasSelected = false;
        Map params = context.getExternalContext().getRequestParameterMap();
        String tag = getSortedTag(context, data);
        if (params.containsKey(tag)) {
            String value = (String) params.get(tag);
            wasSelected = value.equals(column.getId());
            column.getAttributes().put("defaultSort", value);

            
            String sortAcending = (String) params.get(SORTED_SEQUENCE);
            boolean isSortedAcending =  true;
            try {
                isSortedAcending = Boolean.valueOf(sortAcending).booleanValue();
            } catch (Exception e) {
            }
            
            // toggle the sort order flag
            column.setSortAscending(isSortedAcending);

            value = null;
        } else if (column.wasSelected()) {
            wasSelected = true;
        }

        tag = null;
        params = null;

        return wasSelected;
    }

    /**
     * <p>Initiates the component rendering.</p>
     */
    public void encodeBegin(FacesContext context, UIComponent component)
        throws IOException {

        if (context == null || component == null)
            throw new NullPointerException();

        if (!component.isRendered())
            return;

        ResponseWriter writer = context.getResponseWriter();

        UIData data = findDataComponent(component);
        if (data == null)
            throw new NullPointerException(messages.getMessage("headerSorter.invalid"));

        boolean wasSelected =
            wasSelected(context, data, (HeaderSorter) component);

        if (wasSelected) {
            GenericComparator adapter =
                new GenericComparator();
            adapter.setSortAscending(
                ((HeaderSorter) component).isSortAscending());
            adapter.setSortBy(
                ((HeaderSorter) component).getSortBy());

            Collections.sort((List) data.getValue(), adapter);

            adapter = null;
            
            writer.startElement("input", component);
            writer.writeAttribute("type", "hidden", "type");
            writer.writeAttribute("name", this.getSortedTag(context, data), "name");
            writer.writeAttribute("value", ((HeaderSorter) component).getId(), "value");
            writer.write(' ');
        }


        StringBuffer url = new StringBuffer(getActionStr(context));
        url.append('?').append(getSortedTag(context, data)).append("=").append(
            URLEncoder.encode(component.getId(), "UTF-8"))
            .append("&").append(SORTED_SEQUENCE).append("=")
            .append(!((HeaderSorter) component).isSortAscending());
        writer.startElement("a", component);
        writer.writeAttribute("href", context.getExternalContext().encodeActionURL(url.toString()), "href");
        writer.writeText((String) ((HeaderSorter) component).getValue(), "value");

        url = null;
        writer = null;
        data = null;

        super.encodeBegin(context, component);
    }

    /**
     * <p>Completes component rendering.</p>
     */
    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {

        if (context == null || component == null)
            throw new NullPointerException();

        if (!component.isRendered())
            return;

        super.encodeEnd(context, component);

        boolean wasSelected = ((HeaderSorter) component).wasSelected();
        boolean wasSortedAscending =
            ((HeaderSorter) component).isSortAscending();

        ResponseWriter writer = context.getResponseWriter();

        if (wasSelected) {

            StringBuffer url =
                new StringBuffer(
                    context.getExternalContext().getRequestContextPath());

            String title = null;
            if (wasSortedAscending) {
                if (((HeaderSorter) component).getAscImage() != null) {
                    url.append(((HeaderSorter) component).getAscImage());
                } else {
                    url.append(messages.getMessage("headerSorter.ascImage"));
                }
                title = messages.getMessage("headerSorter.ascTitle");
            } else {
                if (((HeaderSorter) component).getDescImage() != null) {
                    url.append(((HeaderSorter) component).getDescImage());
                } else {
                    url.append(messages.getMessage("headerSorter.descImage"));
                }
                title = messages.getMessage("headerSorter.descTitle");
            }

            writer.startElement("img", component);
            writer.writeAttribute("border", "0", "border");
            writer.writeAttribute("src", context.getExternalContext().encodeActionURL(url.toString()), "src");
            writer.writeAttribute("title", title, "title");
            writer.write(' ');
                        
            url = null;
            title = null;

        }
        writer.endElement("a");
        writer = null;

    }
    
}
